-- Hastometer
--
-- Created by Mackenz, Server: Dragonblight
--

-- Constant (non-localised) Data
--
-- Version
HASTOMETER_VERSION			= GetAddOnMetadata("Hastometer", "Version");

-- Enumerated Constants
HASTOMETER_ENUM_EVENTSTATE	= { LOAD=1, LOGIN=2, ENTER=3 };

HASTOMETER_ENUM_SOUNDSTATE	= { NONE=1, QUINTUS=2, HORTATOR=3 };

HASTOMETER_ENUM_AIMED		= { NEVER=1, ALWAYS=2, HASTED=3 };

-- WoW Version/TOC Formats
HASTOMETER_VERSION_WOW		= "^(%d+)%.(%d+)%.(%d+)$";
HASTOMETER_VERSION_TOC		= "%d%02d%02d";

-- Math Constants
HASTOMETER_RADIAN45			= math.cos(math.pi / 4);

-- Equipment Slots
HASTOMETER_RANGEDSLOT		= GetInventorySlotInfo("RangedSlot");

-- Spell Constants
HASTOMETER_SPELL_EPSILON	= 0.0001;

HASTOMETER_SPELL_MIN		= 0.5;
HASTOMETER_SPELL_GCD		= 1.5;
HASTOMETER_SPELL_AIMEDSHOT	= 2.9;
HASTOMETER_SPELL_STEADYSHOT	= 2.0;

-- Saved Variables
Hastometer_Options			= {};

Hastometer_Options_Default	=
{
	Locked				= false;
	Display				= true;
	Rest				= true;
	RestSpeed			= 0;
	RestSound			= HASTOMETER_ENUM_SOUNDSTATE.QUINTUS;
	Battle				= true;
	BattleSpeed			= 0.0;
	BattleHaste			= 20.0;
	BattleSound			= HASTOMETER_ENUM_SOUNDSTATE.QUINTUS;
	Attack				= true;
	AttackSpeed			= 0.0;
	AttackHaste			= 40.0;
	AttackSound			= HASTOMETER_ENUM_SOUNDSTATE.QUINTUS;
	Ramming				= true;
	RammingSound		= HASTOMETER_ENUM_SOUNDSTATE.QUINTUS;
	Aimed				= HASTOMETER_ENUM_AIMED.NEVER;
	AimedSpeed			= 1.8;
	Update30000			= false;
};

-- Global Variables
local	Hastometer_EventState									= HASTOMETER_ENUM_EVENTSTATE.LOAD;

local	Hastometer_VersionWoW									= nil;		-- WoW Version (TOC format)

local	Hastometer_SelfClassType								= nil;		-- Player class

local	Hastometer_Zone											= nil;		-- Zone

local	Hastometer_SilentUpdate									= false;	-- Next update is silent

local	Hastometer_Weapon										= nil;		-- Range weapon
local	Hastometer_SpeedWeapon									= nil;		-- Base weapon speed
local	Hastometer_SpeedCurrent									= nil;		-- Current weapon speed

local	Hastometer_RestSpeedSave								= nil;		-- Rest speed save
local	Hastometer_BattleHasteSave								= nil;		-- Battle haste save
local	Hastometer_AttackHasteSave								= nil;		-- Attack haste save

local	Hastometer_AimedSpeedSave							= nil;		-- Aimed shot speed save


-- Load handler
function Hastometer_OnLoad(self)
	-- hook events
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("PLAYER_LOGOUT")
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_LEAVING_WORLD");
	self:RegisterEvent("UNIT_RANGEDDAMAGE");
	self:RegisterEvent("ZONE_CHANGED");
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA");

	-- hook command handler
	SLASH_HASTOMETER1 = "/hastometer";
	SLASH_HASTOMETER2 = "/ho";

	SlashCmdList["HASTOMETER"] = Hastometer_OnCommand;

	-- echo current version
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(string.format(HASTOMETER_MSG_LOAD, HASTOMETER_VERSION));
	end
end

-- Event handler
function Hastometer_OnEvent(self, event, ...)
	local	arg1 = ...;

	-- determine event
	if		(event == "ADDON_LOADED") then
		Hastometer_ADDON_Load(arg1);
	elseif	(event == "PLAYER_LOGIN") then
		Hastometer_PLAYER_Login();
	elseif	(Hastometer_EventState < HASTOMETER_ENUM_EVENTSTATE.LOGIN) then
		return;
	elseif	(event == "PLAYER_LOGOUT") then
		Hastometer_PLAYER_Logout();
	elseif	(event == "PLAYER_ENTERING_WORLD") then
		Hastometer_PLAYER_Enter();
	elseif	(Hastometer_EventState < HASTOMETER_ENUM_EVENTSTATE.ENTER) then
		return;
	elseif	(event == "PLAYER_LEAVING_WORLD") then
		Hastometer_PLAYER_Leave();
	elseif	(event == "UNIT_RANGEDDAMAGE") then
		Hastometer_RANGE_Update(Hastometer_SilentUpdate);
	elseif	(event == "ZONE_CHANGED") then
		Hastometer_ZONE_Changed();
	elseif	(event == "ZONE_CHANGED_NEW_AREA") then
		Hastometer_ZONE_Changed();
	end
end

-- Command handler
function Hastometer_OnCommand(cmd)
	local	help, ix, msg;

	-- handle command
	if		(cmd == "") then
		-- show help
		help = true;
	elseif	(cmd == HASTOMETER_CMD_HELP) then
		-- show help
		help = true;
	elseif	(cmd == HASTOMETER_CMD_SHOW) then
		-- you *must* be a hunter for the moment
		if (Hastometer_SelfClassType == "HUNTER") then
			-- set display option
			Hastometer_Options.Display = true;

			-- show according to desire
			Hastometer_OnToggle(self, Hastometer_Options.Display);
		else
			-- tell them the bad news
			if (DEFAULT_CHAT_FRAME) then
				DEFAULT_CHAT_FRAME:AddMessage(HASTOMETER_ERR_BADCLASS);
			end
		end
	elseif	(cmd == HASTOMETER_CMD_HIDE) then
		-- reset display option
		Hastometer_Options.Display = false;

		-- hide interface
		Hastometer_OnToggle(self, Hastometer_Options.Display);
	elseif	(cmd == HASTOMETER_CMD_RESETUI) then
		-- reset the position
		Hastometer_Options_ResetUI_OnClick(self);
	else
		-- show help
		help = true;
	end

	-- are we to show help?
	if (help == true) then
		-- hammer through displaying help lines
		ix = 1;

		repeat
			-- get message
			msg = _G["HASTOMETER_HELP_LINE"..ix];

			-- echo message
			if (DEFAULT_CHAT_FRAME and (msg ~= nil)) then
				DEFAULT_CHAT_FRAME:AddMessage(msg);
			end

			-- bump index
			ix = ix + 1;
		until msg == nil;
	end
end

-- UI Toggle handler
function Hastometer_OnToggle(self, visible)
	-- handle toggling of the UI
	if		((visible == false) or ((visible == nil) and Hastometer_Frame:IsVisible())) then
		PlaySound("igMiniMapClose");

		-- hide the hastometer
		Hastometer_Frame:Hide();
	elseif	((visible == true) or ((visible == nil) and not Hastometer_Frame:IsVisible())) then
		PlaySound("igMiniMapOpen");

		-- show the hastometer
		Hastometer_Frame:Show();
	end
end

-- Addon Load handler
function Hastometer_ADDON_Load(addon)
	local	key, value;

	-- is this Hastometer?
	if (addon ~= HASTOMETER_LABEL_HASTOMETER) then
		-- not me
		return;
	end

	-- initialise session
	for key, value in pairs(Hastometer_Options_Default) do
		if (Hastometer_Options[key] == nil) then
			Hastometer_Options[key] = value;
		end
	end

	-- retrieve WoW version
	Hastometer_VersionWoW = Hastometer_GetWoWVersion();

	-- Version Previous -> 30000 update
	--
	-- Calculate haste percentage of battle and attack speeds from their previous absolute values
	if (Hastometer_IsWoWVersion(30000)) then
		-- calculate battle haste from the rest speed and old battle speed
		Hastometer_Options.BattleHaste = math.floor((Hastometer_Options.RestSpeed - Hastometer_Options.BattleSpeed) / Hastometer_Options.RestSpeed * 100 + 0.5);

		-- calculate attack haste from the rest speed and old attack speed
		Hastometer_Options.AttackHaste = math.floor((Hastometer_Options.RestSpeed - Hastometer_Options.AttackSpeed) / Hastometer_Options.RestSpeed * 100 + 0.5);

		-- and mark as having done so
		Hastometer_Options.Updated30000 = true;
	end

	-- initialise options tab
	Hastometer_OptionsFrameVersionFrameVersion:SetText(HASTOMETER_VERSION);

	Hastometer_OptionsFrameLockUI:SetChecked(Hastometer_Options.Locked);

	Hastometer_OptionsFrameRest:SetChecked(Hastometer_Options.Rest);
	Hastometer_OptionsFrameRestSpeed:SetText(Hastometer_Options.RestSpeed);
	Hastometer_OptionsFrameRestSoundNone:SetChecked(Hastometer_Options.RestSound == HASTOMETER_ENUM_SOUNDSTATE.NONE);
	Hastometer_OptionsFrameRestSoundQuintus:SetChecked(Hastometer_Options.RestSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS);
	Hastometer_OptionsFrameRestSoundHortator:SetChecked(Hastometer_Options.RestSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR);

	Hastometer_OptionsFrameBattle:SetChecked(Hastometer_Options.Battle);
	Hastometer_OptionsFrameBattleHaste:SetText(Hastometer_Options.BattleHaste);
	Hastometer_OptionsFrameBattleSoundNone:SetChecked(Hastometer_Options.BattleSound == HASTOMETER_ENUM_SOUNDSTATE.NONE);
	Hastometer_OptionsFrameBattleSoundQuintus:SetChecked(Hastometer_Options.BattleSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS);
	Hastometer_OptionsFrameBattleSoundHortator:SetChecked(Hastometer_Options.BattleSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR);

	Hastometer_OptionsFrameAttack:SetChecked(Hastometer_Options.Attack);
	Hastometer_OptionsFrameAttackHaste:SetText(Hastometer_Options.AttackHaste);
	Hastometer_OptionsFrameAttackSoundNone:SetChecked(Hastometer_Options.AttackSound == HASTOMETER_ENUM_SOUNDSTATE.NONE);
	Hastometer_OptionsFrameAttackSoundQuintus:SetChecked(Hastometer_Options.AttackSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS);
	Hastometer_OptionsFrameAttackSoundHortator:SetChecked(Hastometer_Options.AttackSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR);

	Hastometer_OptionsFrameRamming:SetChecked(Hastometer_Options.Ramming);
	Hastometer_OptionsFrameRammingSoundNone:SetChecked(Hastometer_Options.RammingSound == HASTOMETER_ENUM_SOUNDSTATE.NONE);
	Hastometer_OptionsFrameRammingSoundQuintus:SetChecked(Hastometer_Options.RammingSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS);
	Hastometer_OptionsFrameRammingSoundHortator:SetChecked(Hastometer_Options.RammingSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR);

	Hastometer_OptionsFrameAimedNever:SetChecked(Hastometer_Options.Aimed == HASTOMETER_ENUM_AIMED.NEVER);
	Hastometer_OptionsFrameAimedAlways:SetChecked(Hastometer_Options.Aimed == HASTOMETER_ENUM_AIMED.ALWAYS);
	Hastometer_OptionsFrameAimedHasted:SetChecked(Hastometer_Options.Aimed == HASTOMETER_ENUM_AIMED.HASTED);
	Hastometer_OptionsFrameAimedSpeed:SetText(Hastometer_Options.AimedSpeed);
end

-- Player Login handler
function Hastometer_PLAYER_Login()
	local	_;

	-- bump event state
	Hastometer_EventState = Hastometer_EventState + 1;

	-- get class
	_, Hastometer_SelfClassType = UnitClass("player");

	-- reset zone
	Hastometer_Zone = nil;
end

-- Player Logout handler
function Hastometer_PLAYER_Logout()
	-- previous event state
	Hastometer_EventState = Hastometer_EventState - 1;
end

-- Player Enter World handler
function Hastometer_PLAYER_Enter()
	local	zone, silent;

	-- bump event state
	Hastometer_EventState = Hastometer_EventState + 1;

	-- you *must* be a hunter for the moment
	if (Hastometer_SelfClassType == "HUNTER") then
		-- show according to desire
		Hastometer_OnToggle(self, Hastometer_Options.Display);
	else
		-- always hide
		Hastometer_OnToggle(self, false);
	end

	-- get the zone name (instance name if appropriate)
	zone = GetRealZoneText();

	-- this is a complicated process of ensuring we don't perform a sound announcement when entering/exiting
	-- instances.  Its tough to find this out reliably, but one way to detect that this is happening is
	-- by seeing this silent announcent with the current zone and new zone names the same.  We flip an
	-- addon wide flag to indicate what should be an upcoming silent entry
	silent = Hastometer_Zone == zone;

	-- force silent range update
	Hastometer_RANGE_Update(true);

	-- next update is under this silent watch
	Hastometer_SilentUpdate = silent;
end

-- Player Leave World handler
function Hastometer_PLAYER_Leave()
	-- previous event state
	Hastometer_EventState = Hastometer_EventState - 1;
end

-- Range Update handler
function Hastometer_RANGE_Update(silent)
	local	auto, shottime, shottext, zone;

	-- get current auto speed
	auto = UnitRangedDamage("player");

	-- if rest speed is zero make it current speed
	if (Hastometer_Options.RestSpeed == 0) then
		Hastometer_Options.RestSpeed = auto;
	end

	-- get the zone name (instance name if appropriate)
	zone = GetRealZoneText();

	-- reset silent update flag to make sure we don't do it again
	Hastometer_SilentUpdate = false;

	-- don't urp a change on zone in
	if ((silent == true) or ((auto ~= Hastometer_SpeedCurrent) and (Hastometer_Zone == zone))) then
		-- did our auto speed change?
		local	weapon, msg, music, angle, haste;

		-- get weapon link
		weapon = GetInventoryItemLink("player", HASTOMETER_RANGEDSLOT);

		-- see if our weapon has changed
		if (Hastometer_Weapon ~= weapon) then
			-- get new weapon speed
			Hastometer_Weapon, Hastometer_SpeedWeapon = Hastometer_GetWeaponSpeed(weapon);
		end

		-- no weapon speed means no auto speed
		if (Hastometer_SpeedWeapon == 0) then
			auto = 0;
		end

		-- default is no music
		music = nil;

		-- determine speed type
		if		(auto == 0) then
			-- no range weapon
			msg = HASTOMETER_MSG_NOWEAPON;
		elseif	((auto > Hastometer_Options.RestSpeed) and ((auto - Hastometer_Options.RestSpeed) > HASTOMETER_SPELL_EPSILON)) then
			-- slowed, no sound
			msg = HASTOMETER_MSG_SLOWED;
		elseif ((Hastometer_Options.Rest == true) and (math.abs(auto - Hastometer_Options.RestSpeed) <= HASTOMETER_SPELL_EPSILON)) then
			-- rest
			msg = HASTOMETER_MSG_REST;

			if		(Hastometer_Options.RestSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS) then
				music = "Rest-Quintus";
			elseif	(Hastometer_Options.RestSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR) then
				music = "Rest-Hortator";
			end
		elseif	((Hastometer_Options.Ramming == true) and (auto < Hastometer_Options.AttackSpeed)) then
			-- ramming
			msg = HASTOMETER_MSG_RAMMING;

			if		(Hastometer_Options.RammingSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS) then
				music = "Ramming-Quintus";
			elseif	(Hastometer_Options.RammingSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR) then
				music = "Ramming-Hortator";
			end
		elseif	((Hastometer_Options.Attack == true) and (auto < Hastometer_Options.BattleSpeed) and (auto >= Hastometer_Options.AttackSpeed)) then
			-- attack
			msg = HASTOMETER_MSG_ATTACK;

			if		(Hastometer_Options.AttackSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS) then
				music = "Attack-Quintus";
			elseif	(Hastometer_Options.AttackSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR) then
				music = "Attack-Hortator";
			end
		elseif	((Hastometer_Options.Battle == true) and (auto < Hastometer_Options.RestSpeed) and (auto >= Hastometer_Options.BattleSpeed)) then
			-- battle
			msg = HASTOMETER_MSG_BATTLE;

			if		(Hastometer_Options.BattleSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS) then
				music = "Battle-Quintus";
			elseif	(Hastometer_Options.BattleSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR) then
				music = "Battle-Hortator";
			end
		end

		-- set message
		Hastometer_CommentText:SetText(msg);

		-- if we have music and we are not to be silent, play it
		if ((silent ~= true) and (music ~= nil)) then
			PlaySoundFile("Interface\\AddOns\\Hastometer\\Sounds\\Hastometer-"..music..".mp3");
		end

		-- do we have a weapon?
		if (auto > 0) then
			-- calculate *effective* percent haste if we have a weapon
			haste = math.floor((((Hastometer_SpeedWeapon / auto) - 1.0) * 100.0) + 0.5);
		else
			-- reset haste
			haste = 0;
		end

		-- determine if auto shot has dipped to GCD
		if		(auto == 0) then
			-- and set auto shot to white
			Hastometer_Shot1Bar:SetStatusBarColor(1.0, 1.0, 1.0);
		elseif	(auto > HASTOMETER_SPELL_GCD) then
			-- and set auto shot to green
			Hastometer_Shot1Bar:SetStatusBarColor(0.0, 1.0, 0.0);
		else
			-- and set auto shot to red
			Hastometer_Shot1Bar:SetStatusBarColor(1.0, 0.0, 0.0);
		end

		-- setup auto shot information
		Hastometer_Shot1Text:SetText(string.format(HASTOMETER_AUTO_INFO, auto, haste));

		-- determine shot type settings
		if		(Hastometer_Options.Aimed == HASTOMETER_ENUM_AIMED.NEVER) then
			-- always show steady
			shottime = HASTOMETER_SPELL_STEADYSHOT / (1.0 + haste / 100.0);
			shottext = HASTOMETER_STEADY_INFO;

			-- aimed shot never is white
			Hastometer_Shot2Bar:SetStatusBarColor(1.0, 1.0, 1.0);
		elseif	(Hastometer_Options.Aimed == HASTOMETER_ENUM_AIMED.ALWAYS) then
			-- always show aimed
			shottime = HASTOMETER_SPELL_AIMEDSHOT / (1.0 + haste / 100.0);
			shottext = HASTOMETER_AIMED_INFO;

			-- aimed shot always is cyan
			Hastometer_Shot2Bar:SetStatusBarColor(0.0, 1.0, 1.0);
		else
			-- calculate required shot speed of an aimed shot
			shottime = HASTOMETER_SPELL_AIMEDSHOT / (1.0 + haste / 100.0);

			-- if at hasted threshold, use aimed, else use steady
			if ((haste ~= 0) and (shottime <= Hastometer_Options.AimedSpeed)) then
				-- show aimed when hasted to this rate
				shottext = HASTOMETER_AIMED_INFO;

				-- below aimed shot is orange
				Hastometer_Shot2Bar:SetStatusBarColor(1.0, 0.5, 0.0);
			else
				-- show steady when not hasted to this rate
				shottime = HASTOMETER_SPELL_STEADYSHOT / (1.0 + haste / 100.0);
				shottext = HASTOMETER_STEADY_INFO;

				-- not below aimed shot is still white
				Hastometer_Shot2Bar:SetStatusBarColor(1.0, 1.0, 1.0);
			end
		end

		-- no haste no shottime
		if (haste == 0) then
			shottime = 0;
		end

		-- setup shot information
		Hastometer_Shot2Text:SetText(string.format(shottext, shottime, haste));

		-- calculate indicator position
		if		(auto == 0) then
			-- hide indicator
			angle = 180;
		elseif	(auto > Hastometer_Options.RestSpeed) then
			-- calculate
			angle = (auto - Hastometer_Options.RestSpeed) / Hastometer_Options.RestSpeed * -90 - 90;

			-- make sure slowed auto speed is bounded
			if (angle < -135) then
				-- maxed on left
				angle = -135;
			end
		elseif	(auto < Hastometer_Options.AttackSpeed) then
			-- Ramming.  Calculate with respect to 90 degree offset, applying a minumum to get granularity
			angle = (Hastometer_Options.AttackSpeed - auto) / (Hastometer_Options.AttackSpeed - HASTOMETER_SPELL_MIN) * 90 + 90;

			-- make sure it is bounded
			if (angle > 135) then
				-- maxed on 135
				angle = 135;
			end
		elseif	(auto < Hastometer_Options.BattleSpeed) then
			-- Attack.  Calculate with respect to 0 degree offset
			angle = (Hastometer_Options.BattleSpeed - auto) / (Hastometer_Options.BattleSpeed - Hastometer_Options.AttackSpeed) * 90;

			-- make sure it is bounded
			if (angle > 90) then
				-- maxed on right
				angle = 90;
			end
		elseif	(auto < Hastometer_Options.RestSpeed) then
			-- Battle.  Calculate with respect to -90 degree offset
			angle = (Hastometer_Options.RestSpeed - auto) / (Hastometer_Options.RestSpeed - Hastometer_Options.BattleSpeed) * 90 - 90;

			-- make sure it is bounded
			if (angle > 0) then
				-- maxed on right
				angle = 0;
			end
		else
			-- must be nominal
			angle = -90;
		end

		-- set indicator position
		Hastometer_RotateIndicator(angle);

		-- snarf new auto speed
		Hastometer_SpeedCurrent = auto;
	end

	-- snarf latest zone
	Hastometer_Zone = zone;
end

-- Zone Change handler
function Hastometer_ZONE_Changed()
	-- if we have no weapon, attempt to resolve
	if (Hastometer_Weapon == nil) then
		-- attempt to resolve weapon
		Hastometer_RANGE_Update(true);
	else
		-- get the zone name (instance name if appropriate)
		Hastometer_Zone = GetRealZoneText();

		-- reset silent update flag to make sure we don't do it again
		Hastometer_SilentUpdate = false;
	end
end

-- Options User Interface handler
function Hastometer_Options_OnClick(self)
	-- show/hide options window as appropriate
	if (Hastometer_OptionsFrame:IsVisible()) then
		-- hide options window
		HideUIPanel(Hastometer_OptionsFrame);
	else
		-- show options window
		ShowUIPanel(Hastometer_OptionsFrame);
	end	
end

-- Options User Interface handler
function Hastometer_Option_Close_OnClick(self)
	-- hide options window
	HideUIPanel(Hastometer_OptionsFrame);
end

-- Option Lock UI Toggle handler
function Hastometer_Options_LockUI_OnToggle(self)
	-- this is easy
	Hastometer_Options.Locked = (Hastometer_OptionsFrameLockUI:GetChecked() ~= nil);
end

-- Option Reset UI Click handler
function Hastometer_Options_ResetUI_OnClick(self)
	-- reset UI, the user has lost the window off the screen.  Reset the anchors to center
	Hastometer_Frame:ClearAllPoints();
	Hastometer_Frame:SetPoint("CENTER");
end

-- Option Use Rest Speed Toggle handler
function Hastometer_Options_Rest_OnToggle(self)
	-- this is easy
	Hastometer_Options.Rest = (Hastometer_OptionsFrameRest:GetChecked() ~= nil);
end

-- Option Rest Speed Focus Gained handler
function Hastometer_Options_RestSpeed_OnFocusGained(self)
	-- save the current value so we can detect changes
	Hastometer_RestSpeedSave = Hastometer_Options.RestSpeed;
end

-- Option Rest Speed Focus Lost handler
function Hastometer_Options_RestSpeed_OnFocusLost(self)
	-- retrieve value
	Hastometer_Options.RestSpeed = tonumber(Hastometer_OptionsFrameRestSpeed:GetText());

	-- must be a valid number
	if (Hastometer_Options.RestSpeed == nil) then
		-- restore and reemit
		Hastometer_Options.RestSpeed = Hastometer_RestSpeedSave;

		-- update
		Hastometer_OptionsFrameRestSpeed:SetText(Hastometer_Options.RestSpeed);
	end

	-- and reset the Rest speed save
	Hastometer_RestSpeedSave = nil;
	
	-- recalculate battle and attack speeds
	Hastometer_Options.BattleSpeed = Hastometer_Options.RestSpeed - Hastometer_Options.RestSpeed * Hastometer_Options.BattleHaste / 100;
	Hastometer_Options.AttackSpeed = Hastometer_Options.RestSpeed - Hastometer_Options.RestSpeed * Hastometer_Options.AttackHaste / 100;
	
	-- update
	Hastometer_RANGE_Update(true);
end

-- Option Rest Speed Set handler
function Hastometer_Options_RestSpeedSet_OnClick(self)
	-- set rest speed from current speed
	Hastometer_Options.RestSpeed = Hastometer_SpeedCurrent;

	-- update
	Hastometer_OptionsFrameRestSpeed:SetText(Hastometer_Options.RestSpeed);
	
	-- recalculate battle and attack speeds
	Hastometer_Options.BattleSpeed = Hastometer_Options.RestSpeed - Hastometer_Options.RestSpeed * Hastometer_Options.BattleHaste / 100;
	Hastometer_Options.AttackSpeed = Hastometer_Options.RestSpeed - Hastometer_Options.RestSpeed * Hastometer_Options.AttackHaste / 100;
	
	-- update
	Hastometer_RANGE_Update(true);
end

-- Option Play Rest Sound Click handler
function Hastometer_Options_RestSpeedPlay_OnClick(self)
	local	music;

	-- play rest announcement
	if		(Hastometer_Options.RestSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS) then
		music = "Rest-Quintus";
	elseif	(Hastometer_Options.RestSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR) then
		music = "Rest-Hortator";
	else
		music = nil;
	end

	-- if we have music, play it
	if (music ~= nil) then
		PlaySoundFile("Interface\\AddOns\\Hastometer\\Sounds\\Hastometer-"..music..".mp3");
	end
end

-- Option Use Battle Haste Toggle handler
function Hastometer_Options_Battle_OnToggle(self)
	-- this is easy
	Hastometer_Options.Battle = (Hastometer_OptionsFrameBattle:GetChecked() ~= nil);
end

-- Option Battle Haste Focus Gained handler
function Hastometer_Options_BattleHaste_OnFocusGained(self)
	-- save the current value so we can detect changes
	Hastometer_BattleHasteSave = Hastometer_Options.BattleHaste;
end

-- Option Battle Haste Focus Lost handler
function Hastometer_Options_BattleHaste_OnFocusLost(self)
	-- retrieve value
	Hastometer_Options.BattleHaste = tonumber(Hastometer_OptionsFrameBattleHaste:GetText());

	-- must be a valid number
	if (Hastometer_Options.BattleHaste == nil) then
		-- restore and reemit
		Hastometer_Options.BattleHaste = Hastometer_BattleHasteSave;

		-- update
		Hastometer_OptionsFrameBattleHaste:SetText(Hastometer_Options.BattleHaste);
	end

	-- and battle the Battle haste save
	Hastometer_BattleHasteSave = nil;
	
	-- recalculate battle speed
	Hastometer_Options.BattleSpeed = Hastometer_Options.RestSpeed - Hastometer_Options.RestSpeed * Hastometer_Options.BattleHaste / 100;
	
	-- update
	Hastometer_RANGE_Update(true);
end

-- Option Battle Haste Set handler
function Hastometer_Options_BattleHasteSet_OnClick(self)
	-- set battle haste from current speed and reset speed
	Hastometer_Options.BattleHaste = math.floor((Hastometer_Options.RestSpeed - Hastometer_SpeedCurrent) / Hastometer_Options.RestSpeed * 100 + 0.5);

	-- update
	Hastometer_OptionsFrameBattleHaste:SetText(Hastometer_Options.BattleHaste);
	
	-- recalculate battle speed
	Hastometer_Options.BattleSpeed = Hastometer_Options.RestSpeed - Hastometer_Options.RestSpeed * Hastometer_Options.BattleHaste / 100;
	
	-- update
	Hastometer_RANGE_Update(true);
end

-- Option Play Battle Sound Click handler
function Hastometer_Options_BattleHastePlay_OnClick(self)
	local	music;

	-- play battle announcement
	if		(Hastometer_Options.BattleSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS) then
		music = "Battle-Quintus";
	elseif	(Hastometer_Options.BattleSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR) then
		music = "Battle-Hortator";
	else
		music = nil;
	end

	-- if we have music, play it
	if (music ~= nil) then
		PlaySoundFile("Interface\\AddOns\\Hastometer\\Sounds\\Hastometer-"..music..".mp3");
	end
end

-- Option Use Attack Haste Toggle handler
function Hastometer_Options_Attack_OnToggle(self)
	-- this is easy
	Hastometer_Options.Attack = (Hastometer_OptionsFrameAttack:GetChecked() ~= nil);
end

-- Option Attack Haste Focus Gained handler
function Hastometer_Options_AttackHaste_OnFocusGained(self)
	-- save the current value so we can detect changes
	Hastometer_AttackHasteSave = Hastometer_Options.AttackHaste;
end

-- Option Attack Haste Focus Lost handler
function Hastometer_Options_AttackHaste_OnFocusLost(self)
	-- retrieve value
	Hastometer_Options.AttackHaste = tonumber(Hastometer_OptionsFrameAttackHaste:GetText());

	-- must be a valid number
	if (Hastometer_Options.AttackHaste == nil) then
		-- restore and reemit
		Hastometer_Options.AttackHaste = Hastometer_AttackHasteSave;

		-- update
		Hastometer_OptionsFrameAttackHaste:SetText(Hastometer_Options.AttackHaste);
	end

	-- and attack the Attack haste save
	Hastometer_AttackHasteSave = nil;
	
	-- recalculate attack speed
	Hastometer_Options.AttackSpeed = Hastometer_Options.RestSpeed - Hastometer_Options.RestSpeed * Hastometer_Options.AttackHaste / 100;
	
	-- update
	Hastometer_RANGE_Update(true);
end

-- Option Attack Haste Set handler
function Hastometer_Options_AttackHasteSet_OnClick(self)
	-- set attack haste from current speed and reset speed
	Hastometer_Options.AttackHaste = math.floor((Hastometer_Options.RestSpeed - Hastometer_SpeedCurrent) / Hastometer_Options.RestSpeed * 100 + 0.5);

	-- update
	Hastometer_OptionsFrameAttackHaste:SetText(Hastometer_Options.AttackHaste);
	
	-- recalculate attack speed
	Hastometer_Options.AttackSpeed = Hastometer_Options.RestSpeed - Hastometer_Options.RestSpeed * Hastometer_Options.AttackHaste / 100;
	
	-- update
	Hastometer_RANGE_Update(true);
end

-- Option Play Attack Sound Click handler
function Hastometer_Options_AttackHastePlay_OnClick(self)
	local	music;

	-- play attack announcement
	if		(Hastometer_Options.AttackSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS) then
		music = "Attack-Quintus";
	elseif	(Hastometer_Options.AttackSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR) then
		music = "Attack-Hortator";
	else
		music = nil;
	end

	-- if we have music, play it
	if (music ~= nil) then
		PlaySoundFile("Interface\\AddOns\\Hastometer\\Sounds\\Hastometer-"..music..".mp3");
	end
end

-- Option Use Ramming Speed Toggle handler
function Hastometer_Options_Ramming_OnToggle(self)
	-- this is easy
	Hastometer_Options.Ramming = (Hastometer_OptionsFrameRamming:GetChecked() ~= nil);
end

-- Option Play Ramming Sound Click handler
function Hastometer_Options_RammingSpeedPlay_OnClick(self)
	local	music;

	-- play ramming announcement
	if		(Hastometer_Options.RammingSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS) then
		music = "Ramming-Quintus";
	elseif	(Hastometer_Options.RammingSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR) then
		music = "Ramming-Hortator";
	else
		music = nil;
	end

	-- if we have music, play it
	if (music ~= nil) then
		PlaySoundFile("Interface\\AddOns\\Hastometer\\Sounds\\Hastometer-"..music..".mp3");
	end
end

-- Option Sound Radio Button Click handler
function Hastometer_Options_SoundRadioButton_OnClick(self, id)
	-- determine which sound set to set
	if		((id >= 1) and (id <= 3)) then
		Hastometer_Options.RestSound = id;

		-- and set correct radio
		Hastometer_OptionsFrameRestSoundNone:SetChecked(Hastometer_Options.RestSound == HASTOMETER_ENUM_SOUNDSTATE.NONE);
		Hastometer_OptionsFrameRestSoundQuintus:SetChecked(Hastometer_Options.RestSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS);
		Hastometer_OptionsFrameRestSoundHortator:SetChecked(Hastometer_Options.RestSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR);
	elseif	((id >= 4) and (id <= 6)) then
		Hastometer_Options.BattleSound = id - 3;

		-- and set correct radio
		Hastometer_OptionsFrameBattleSoundNone:SetChecked(Hastometer_Options.BattleSound == HASTOMETER_ENUM_SOUNDSTATE.NONE);
		Hastometer_OptionsFrameBattleSoundQuintus:SetChecked(Hastometer_Options.BattleSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS);
		Hastometer_OptionsFrameBattleSoundHortator:SetChecked(Hastometer_Options.BattleSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR);
	elseif	((id >= 7) and (id <= 9)) then
		Hastometer_Options.AttackSound = id - 6;

		-- and set correct radio
		Hastometer_OptionsFrameAttackSoundNone:SetChecked(Hastometer_Options.AttackSound == HASTOMETER_ENUM_SOUNDSTATE.NONE);
		Hastometer_OptionsFrameAttackSoundQuintus:SetChecked(Hastometer_Options.AttackSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS);
		Hastometer_OptionsFrameAttackSoundHortator:SetChecked(Hastometer_Options.AttackSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR);
	elseif	((id >= 10) and (id <= 12)) then
		Hastometer_Options.RammingSound = id - 9;

		-- and set correct radio
		Hastometer_OptionsFrameRammingSoundNone:SetChecked(Hastometer_Options.RammingSound == HASTOMETER_ENUM_SOUNDSTATE.NONE);
		Hastometer_OptionsFrameRammingSoundQuintus:SetChecked(Hastometer_Options.RammingSound == HASTOMETER_ENUM_SOUNDSTATE.QUINTUS);
		Hastometer_OptionsFrameRammingSoundHortator:SetChecked(Hastometer_Options.RammingSound == HASTOMETER_ENUM_SOUNDSTATE.HORTATOR);
	end	
end

-- Option Shot Radio Button Click handler
function Hastometer_Options_ShotRadioButton_OnClick(self, id)
	-- determine which shot set to set
	if		((id >= 1) and (id <= 3)) then
		Hastometer_Options.Aimed = id;

		-- and set correct radio
		Hastometer_OptionsFrameAimedNever:SetChecked(Hastometer_Options.Aimed == HASTOMETER_ENUM_AIMED.NEVER);
		Hastometer_OptionsFrameAimedAlways:SetChecked(Hastometer_Options.Aimed == HASTOMETER_ENUM_AIMED.ALWAYS);
		Hastometer_OptionsFrameAimedHasted:SetChecked(Hastometer_Options.Aimed == HASTOMETER_ENUM_AIMED.HASTED);
	
		-- update
		Hastometer_RANGE_Update(true);
	end
end

-- Option Aimed Speed Focus Gained handler
function Hastometer_Options_AimedSpeed_OnFocusGained(self)
	-- save the current value so we can detect changes
	Hastometer_AimedSpeedSave = Hastometer_Options.AimedSpeed;
end

-- Option Aimed Speed Focus Lost handler
function Hastometer_Options_AimedSpeed_OnFocusLost(self)
	-- retrieve value
	Hastometer_Options.AimedSpeed = tonumber(Hastometer_OptionsFrameAimedSpeed:GetText());

	-- must be a valid number
	if (Hastometer_Options.AimedSpeed == nil) then
		-- restore and reemit
		Hastometer_Options.AimedSpeed = Hastometer_AimedSpeedSave;

		-- update
		Hastometer_OptionsFrameAimedSpeed:SetText(Hastometer_Options.AimedSpeed);
	end

	-- and reset the Aimed shot speed save
	AimedSpeed = nil;
	
	-- update
	Hastometer_RANGE_Update(true);
end

-- Get WoW Version in TOC Format
function Hastometer_GetWoWVersion()
	local	versionWoW, versionTOC;
	local	_, major, minor, patch;

	-- retrieve WoW version
	versionWoW = GetBuildInfo();

	-- tease apart the version into components
	_, _, major, minor, patch = string.find(versionWoW, HASTOMETER_VERSION_WOW);

	-- ensure we are valid
	if ((major == nil) or (minor == nil) or (patch == nil)) then
		-- use default
		major = 0;
		minor = 0;
		patch = 0;
	else
		-- convert to numeric
		major = tonumber(major);
		minor = tonumber(minor);
		patch = tonumber(patch);
	end

	-- convert to TOC format
	versionTOC = string.format(HASTOMETER_VERSION_TOC, major, minor, patch);

	return tonumber(versionTOC);
end

-- Test WoW Version
function Hastometer_IsWoWVersion(version)
	-- are we on a PTR or WoW's version is equal or higher than than the test?
	if ((Hastometer_VersionWoW < 10000) or (Hastometer_VersionWoW >= version)) then
		-- return the requisite update value in the options
		return (Hastometer_Options["Updated"..version] == nil) or (Hastometer_Options["Updated"..version] == false);
	else
		-- not our time
		return false;
	end
end

-- Indicator Rotation handler
function Hastometer_RotateIndicator(angle)
	local	radians;
	local	x1, y1, x2, y2;
	local	swatch;

	-- do we have a valid angle?
	if (angle ~= 180) then
		-- rotate the indicator by the required angle.  Calculate angle in radians
		radians = math.pi * (45 - angle) / 180;

		-- calculate texture coordinates
		x1 = HASTOMETER_RADIAN45 * math.cos(radians);
		y1 = HASTOMETER_RADIAN45 * math.sin(radians);

		x2 = -y1;
		y2 = x1;

		-- rotate indicator
		Hastometer_Indicator:SetTexCoord((0.5 - x1), (0.65625 - y1), (0.5 + x2), 
										 (0.65625 + y2), (0.5 - x2), (0.65625 - y2), 
										 (0.5 + x1), (0.65625 + y1));

		-- and show indicator
		Hastometer_Indicator:Show();

		-- get the swatch color
		swatch = HASTOMETER_SWATCH[math.floor(((angle / 135) * 63) + 64.5)];

		-- and color the comment bar
		Hastometer_CommentBar:SetStatusBarColor((swatch.r / 255), (swatch.g / 255), (swatch.b / 255));
	else
		-- hide indicator
		Hastometer_Indicator:Hide();

		-- and set swatch to white
		Hastometer_CommentBar:SetStatusBarColor(1.0, 1.0, 1.0);
	end
end

-- Retrieve Weapon Speed handler
function Hastometer_GetWeaponSpeed(link)
	local	ix, text;
	local	_, speed;

	-- no link means we are done
	if ((link == nil) or (GetItemInfo(link) == nil)) then
		return nil, 0.0;
	end

	-- set tooltip owner
	Hastometer_Tooltip:SetOwner(UIParent, "ANCHOR_NONE");

	-- process the item for weapon speed by getting the game tooltip to parse prepare the data
	Hastometer_Tooltip:SetHyperlink(link);

	-- hammer through the tooltip text items looking for any class restrictions
	for ix=1, 30, 1 do
		-- get left text field (the prompt)
		field = _G["Hastometer_TooltipTextRight"..ix];

		if ((field ~= nil) and field:IsShown()) then
			text = field:GetText();

			-- if we have a line break, end processing
			if ((text ~= nil) and (string.sub(text, 1, 1) == "\n")) then
				text = nil;

				ix = 30;
			end
		else
			text = nil;
		end

		-- do we have text?
		if (text ~= nil) then
			-- test for weapon speed
			_, _, speed = string.find(text, HASTOMETER_SPEED_PATTERN);

			-- convert to float
			speed = tonumber(speed);

			-- if we found speed we are done
			if (speed ~= nil) then
				ix = 30
			end
		end
	end

	-- make sure we have a speed
	if (speed == nil) then
		speed = 0.0;
	end

	return link, speed;
end

-- Swatch Color Lookup Table
HASTOMETER_SWATCH = {

	{ r=  0, g=  0, b=255 },
	{ r=  0, g=  6, b=248 },
	{ r=  0, g= 13, b=242 },
	{ r=  0, g= 19, b=236 },
	{ r=  0, g= 26, b=229 },
	{ r=  0, g= 33, b=223 },
	{ r=  0, g= 39, b=217 },
	{ r=  0, g= 46, b=210 },
	{ r=  0, g= 52, b=204 },
	{ r=  0, g= 59, b=198 },
	{ r=  0, g= 66, b=192 },
	{ r=  0, g= 72, b=185 },
	{ r=  0, g= 79, b=179 },
	{ r=  0, g= 85, b=173 },
	{ r=  0, g= 92, b=166 },
	{ r=  0, g= 99, b=160 },
	{ r=  0, g=105, b=154 },
	{ r=  0, g=112, b=147 },
	{ r=  0, g=118, b=141 },
	{ r=  0, g=125, b=135 },
	{ r=  0, g=179, b=129 },
	{ r=  0, g=255, b= 76 },
	{ r=  2, g=252, b=  0 },
	{ r=  5, g=250, b=  0 },
	{ r=  7, g=247, b=  0 },
	{ r= 10, g=245, b=  0 },
	{ r= 12, g=242, b=  0 },
	{ r= 15, g=240, b=  0 },
	{ r= 17, g=237, b=  0 },
	{ r= 20, g=235, b=  0 },
	{ r= 22, g=232, b=  0 },
	{ r= 25, g=230, b=  0 },
	{ r= 27, g=227, b=  0 },
	{ r= 30, g=225, b=  0 },
	{ r= 32, g=223, b=  0 },
	{ r= 35, g=220, b=  0 },
	{ r= 37, g=218, b=  0 },
	{ r= 40, g=215, b=  0 },
	{ r= 42, g=213, b=  0 },
	{ r= 45, g=210, b=  0 },
	{ r= 47, g=208, b=  0 },
	{ r= 50, g=205, b=  0 },
	{ r= 52, g=203, b=  0 },
	{ r= 55, g=200, b=  0 },
	{ r= 57, g=198, b=  0 },
	{ r= 60, g=195, b=  0 },
	{ r= 62, g=193, b=  0 },
	{ r= 65, g=191, b=  0 },
	{ r= 67, g=188, b=  0 },
	{ r= 70, g=186, b=  0 },
	{ r= 72, g=183, b=  0 },
	{ r= 75, g=181, b=  0 },
	{ r= 77, g=178, b=  0 },
	{ r= 80, g=176, b=  0 },
	{ r= 82, g=173, b=  0 },
	{ r= 85, g=171, b=  0 },
	{ r= 87, g=168, b=  0 },
	{ r= 90, g=166, b=  0 },
	{ r= 92, g=163, b=  0 },
	{ r= 95, g=161, b=  0 },
	{ r= 97, g=159, b=  0 },
	{ r=100, g=156, b=  0 },
	{ r=102, g=154, b=  0 },
	{ r=105, g=151, b=  0 },
	{ r=107, g=149, b=  0 },
	{ r=110, g=146, b=  0 },
	{ r=112, g=144, b=  0 },
	{ r=115, g=141, b=  0 },
	{ r=117, g=139, b=  0 },
	{ r=120, g=136, b=  0 },
	{ r=122, g=134, b=  0 },
	{ r=125, g=131, b=  0 },
	{ r=127, g=129, b=  0 },
	{ r=129, g=127, b=  0 },
	{ r=131, g=124, b=  0 },
	{ r=134, g=122, b=  0 },
	{ r=136, g=119, b=  0 },
	{ r=139, g=117, b=  0 },
	{ r=141, g=114, b=  0 },
	{ r=144, g=112, b=  0 },
	{ r=146, g=110, b=  0 },
	{ r=148, g=107, b=  0 },
	{ r=151, g=105, b=  0 },
	{ r=153, g=102, b=  0 },
	{ r=156, g=100, b=  0 },
	{ r=158, g= 98, b=  0 },
	{ r=161, g= 95, b=  0 },
	{ r=163, g= 93, b=  0 },
	{ r=166, g= 90, b=  0 },
	{ r=168, g= 88, b=  0 },
	{ r=170, g= 86, b=  0 },
	{ r=173, g= 83, b=  0 },
	{ r=175, g= 81, b=  0 },
	{ r=178, g= 78, b=  0 },
	{ r=180, g= 76, b=  0 },
	{ r=183, g= 73, b=  0 },
	{ r=185, g= 71, b=  0 },
	{ r=187, g= 69, b=  0 },
	{ r=190, g= 66, b=  0 },
	{ r=192, g= 64, b=  0 },
	{ r=195, g= 61, b=  0 },
	{ r=197, g= 59, b=  0 },
	{ r=200, g= 57, b=  0 },
	{ r=202, g= 54, b=  0 },
	{ r=205, g= 52, b=  0 },
	{ r=207, g= 49, b=  0 },
	{ r=209, g= 47, b=  0 },
	{ r=212, g= 45, b=  0 },
	{ r=214, g= 42, b=  0 },
	{ r=217, g= 40, b=  0 },
	{ r=219, g= 37, b=  0 },
	{ r=222, g= 35, b=  0 },
	{ r=224, g= 32, b=  0 },
	{ r=226, g= 30, b=  0 },
	{ r=229, g= 28, b=  0 },
	{ r=231, g= 25, b=  0 },
	{ r=234, g= 23, b=  0 },
	{ r=236, g= 20, b=  0 },
	{ r=239, g= 18, b=  0 },
	{ r=241, g= 16, b=  0 },
	{ r=244, g= 13, b=  0 },
	{ r=246, g= 11, b=  0 },
	{ r=248, g=  8, b=  0 },
	{ r=251, g=  6, b=  0 },
	{ r=253, g=  4, b=  0 },
	{ r=255, g=  1, b=  0 },
	{ r=255, g=  0, b=  0 }

};
